;***********************************************************
;   
;      This code will generate a direct-sequence
;      spread-spectrum digital serial signal on a
;      PIC17C42 microcontroller.
;
;      PN Generator will generate a sequence of 31 bit 
;      bit and the program will transmit the serial bit
;      at 9600 bps => PN clk  = 9600 x 31 = 290.625kbps
;                             => 3.36 us/bit
;
;      The PIC17C42 microcontroller has a 250ns cycle time
;      and most instruction will take only 1 cycle per 
;      instruction. Approximately, the PIC can handled up 
;      to 13 instructions per bit that it is transmitted.
;
;
; NOTE! THIS CHIP USES BYTE ADDRESSING FOR ITS DATA 
;       REGISTER FILE, AND 2 BYTE, WORD ADDRESSING FOR
;       ITS PROGRAM MEMORY.  THEREFORE, ONE MUST SET 
;       WORD LENGTH (WDLN) TO 1 WHEN DEFINING DATA MEMORY,
;       AND 2 WHEN DEFINING PROGRAM MEMORY.
;       
;***********************************************************

; The following file tell the assembler to assemble 
; source file using PIC1742 instruction set.
	CPU     "c:\rf-ss\cross16\PIC1742.TBL"

; INT16 is the standard format for file transfer for the serial 
; loader LD17 
	HOF     "INT16"

; Register file
	HEX     "OFF"   ;Turn hex output off
	ORG     0x0
	WDLN    1       ;1 byte data for data memory


; Unbanked
INDF0:  DFS     1       ;Uses F1 to address data memory
FSR0:   DFS     1       ;inderect data memory address pointer 0
PCL:    DFS     1       ;Low order 8 bits of PC
PCLATH: DFS     1       ;Holding register for upper 8 bits of PC
ALUSTA: DFS     1       ;ALU Status
RTCSTA: DFS     1       ;Real Time Clock Status
CPUSTA: DFS     1       ;CPU Status
INTSTA: DFS     1       ;Interrupt Status
INDF1:  DFS     1       ;Uses contents of F9 to address data memory
FSR1:   DFS     1       ;Indirect data memory address pointer 1
W:      DFS     1       ;W register
RTCCL:  DFS     1       ;Real time clock counter LS byte
RTCCH:  DFS     1       ;Real time clock counter MS byte
TBLPTRL: DFS    1       ;Low byte of program memory table pointer
TBLPTRH: DFS    1       ;High byte of program memory table pointer
BSR:    DFS     1       ;Bank select register
;
; Bank 0
PORTA:  DFS     1       ;Port A
DDRB:   DFS     1       ;Data direction register for Port B
PORTB:  DFS     1       ;Port B data latch
RCSTA:  DFS     1       ;Serial Port Receive Status
RCREG:  DFS     1       ;Serial Port Receive Register
TXSTA:  DFS     1       ;Serial Port Transmit Statu
TXREG:  DFS     1       ;Serial Port Transmit Register
SPBRG:  DFS     1       ;Baud Rate Generator
;
; All Banks start at 10h
; Bank 1
DDRC:   EQU     10h     ;Data Direction Register for Port C
PORTC:  EQU     11h     ;Port C Data Latch
DDRD:   EQU     12h     ;Data Direction Register for Port D
PORTD:  EQU     13h     ;Port D Data Latch
DDRE:   EQU     14h     ;Data Direction Register for Port E
PORTE:  EQU     15h     ;Port E Data Latch
PIR:    EQU     16h     ;Peripheral Interrupt Request
PIE:    EQU     17h     ;Peripheral Interrupt Enable
;
; Bank 2
TMR1:   EQU     10h     ;Timer 1
TMR2:   EQU     11h     ;Timer 2
TMR3L:  EQU     12h     ;Timer 3 Low byte
TMR3H:  EQU     13h     ;Timer 3 High byte
PR1:    EQU     14h     ;Timer 1 Period Register
PR2:    EQU     15h     ;Timer 2 Period Register
PR3L:   EQU     16h     ;Timer 3 Period Register, Low byte
PR3H:   EQU     17h     ;Timer 3 Period Register, High byte
CA1L:   EQU     16h     ;Capture 1 register, Low byte
CA1H:   EQU     17h     ;Capture 1 register, High byte
;
; Bank 3
PW1DCL: EQU     10h     ;
PW2DCL: EQU     11h     ;
PW1DCH: EQU     12h     ;
PW2DCH: EQU     13h     ;
CA2L:   EQU     14h     ;Capture 2 low byte
CA2H:   EQU     15h     ;Capture 2 high byte
TCON1:  EQU     16h     ;
TCON2:  EQU     17h     ;
;
;User registers
User1:  DFS     1       ;
User2:  DFS     1       ;
;
; etc.
;

; Code segment
	ORG     0       ;Begining of Program Memory
	HEX     "ON"    ;Turn hex output on
	WDLN    2       ;Set program word length to 2 bytes

	goto    init1      ; jump to begining of program at 100h   
                           ; skip address 0x00 to 0xff to avoid 
                           ; possible address confict

	DFS     8       ;Reset Vector
	DFS     8       ;INT Pin Interrupt Vector
	DFS     8       ;RTTC Timer Interrupt Vector
	DFS     8       ;RT Pin Interrupt Vector
	DFS     8       ;Peripheral Interrupt Vector

; EXAMPLES OF PIC17C42 INSTRUCTIONS FOLLOW:
;
; Program content starts at 0x100
	org     0100h

; PN-Code => 0000 0110 0101 1011 1101 0100 0100 111
;
; transfer PN-Code from program memory to data memory for 
; high speed purpose
;
; non-inverted PN-Code starts at 20h to 3Eh in data memory
; inverted PN-Code starts at 40h to 5Eh in data memory
;

     init1:
	    movlw 0x0   ; for non-inverted sequence with 0  
	    movwf 0x20  ; bit 0
	    movwf 0x21  ; bit 1
	    movwf 0x22  ; bit 2
	    movwf 0x23  ; bit 3

	    movwf 0x24  ; bit 4
	    movwf 0x27  ; bit 7

	    movwf 0x28  ; bit 8
	    movwf 0x2A  ; bit 10

	    movwf 0x2D  ; bit 13

	    movwf 0x32  ; bit 18

	    movwf 0x34  ; bit 20
	    movwf 0x36  ; bit 22
	    movwf 0x37  ; bit 23

	    movwf 0x38  ; bit 24
	    movwf 0x3A  ; bit 26
	    movwf 0x3B  ; bit 27
	   
	    movlw 0x0F    ; for non-inverted sequence with 1
	    movwf 0x25   ; bit 5   
	    movwf 0x26   ; bit 6 
  
	    movwf 0x29   ; bit 9   
	    movwf 0x2B   ; bit 11
   
	    movwf 0x2C   ; bit 12   
	    movwf 0x2E   ; bit 14   
	    movwf 0x2F   ; bit 15
   
	    movwf 0x30   ; bit 16   
	    movwf 0x31   ; bit 17   
	    movwf 0x33   ; bit 19
   
	    movwf 0x35   ; bit 21   

	    movwf 0x39   ; bit 25   
  
	    movwf 0x3C   ; bit 28   
	    movwf 0x3D   ; bit 29   
	    movwf 0x3E   ; bit 30   
 
	    
    init2:  movlw 0xF   ; for inverted sequence with 1  
	    movwf 0x40  ; bit 0
	    movwf 0x41  ; bit 1
	    movwf 0x42  ; bit 2
	    movwf 0x43  ; bit 3

	    movwf 0x44  ; bit 4
	    movwf 0x47  ; bit 7

	    movwf 0x48  ; bit 8
	    movwf 0x4A  ; bit 10

	    movwf 0x4D  ; bit 13

	    movwf 0x52  ; bit 18

	    movwf 0x54  ; bit 20
	    movwf 0x56  ; bit 22
	    movwf 0x57  ; bit 23

	    movwf 0x58  ; bit 24
	    movwf 0x5A  ; bit 26
	    movwf 0x5B  ; bit 27
	   
	    movlw 0x0    ; for inverted sequence with 0
	    movwf 0x45   ; bit 5   
	    movwf 0x46   ; bit 6 
  
	    movwf 0x49   ; bit 9   
	    movwf 0x4B   ; bit 11
   
	    movwf 0x4C   ; bit 12   
	    movwf 0x4E   ; bit 14   
	    movwf 0x4F   ; bit 15
   
	    movwf 0x50   ; bit 16   
	    movwf 0x51   ; bit 17   
	    movwf 0x53   ; bit 19
   
	    movwf 0x55   ; bit 21   

	    movwf 0x59   ; bit 25   
  
	    movwf 0x5C   ; bit 28   
	    movwf 0x5D   ; bit 29   
	    movwf 0x5E   ; bit 30   


; Generate Sequence in Port A
; This code segment is intended to test the parallel port at the
; highest possible rate. 9600 bps xor with 31 bit PN-code is not
; yet implemented 

      main:         
     	    lcall ninv  ; jump subroutine to ninv PN-Code     
	    lcall inverted ; jump subroutine to inv PN-Code

	    lcall portA_test; 

       end: goto end   ; loop forever

portA_test: ; only bit 2 & 3 are able to be configured as input
	    ; or output, other pins are input only

	    movlw 0x00  ; bit 2@20khz, bit 3=1
	    movwf portA 
	    nop
	    nop
	    nop
	    nop
	    nop
	    nop
	    nop
	    nop
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    movlw 0x0C
	    movwf portA
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    lcall delay
	    nop
	    nop
	    nop
	    nop
	    nop
	    nop
	    goto  portA_test

	    return

    
    serial: 
	   
	   return           


     delay:     ; should have a delay of 10 cycle including the call inst. 
	   nop
	   nop
	   nop
	   nop
	   nop
	   nop
	   return



      ninv:
	   movfp 0x20, portA    ; start of non-inverted sequence
	   movfp 0x21, portA
	   movfp 0x22, portA
	   movfp 0x23, portA
	   movfp 0x24, portA
	   movfp 0x25, portA
	   movfp 0x26, portA
	   movfp 0x27, portA
	   movfp 0x28, portA
	   movfp 0x29, portA
	   movfp 0x2A, portA
	   movfp 0x2B, portA
	   movfp 0x2C, portA
	   movfp 0x2D, portA
	   movfp 0x2E, portA
	   movfp 0x2F, portA
	   movfp 0x30, portA
	   movfp 0x31, portA
	   movfp 0x32, portA
	   movfp 0x33, portA
	   movfp 0x34, portA
	   movfp 0x35, portA
	   movfp 0x36, portA
	   movfp 0x37, portA
	   movfp 0x38, portA
	   movfp 0x39, portA
	   movfp 0x3A, portA
	   movfp 0x3B, portA
	   movfp 0x3C, portA
	   movfp 0x3D, portA
	   movfp 0x3E, portA
 
	    return         


    inverted:
	   movfp 0x40, portA    ; start of inverted sequence
	   movfp 0x41, portA
	   movfp 0x42, portA
	   movfp 0x43, portA
	   movfp 0x44, portA
	   movfp 0x45, portA
	   movfp 0x46, portA
	   movfp 0x47, portA
	   movfp 0x48, portA
	   movfp 0x49, portA
	   movfp 0x4A, portA
	   movfp 0x4B, portA
	   movfp 0x4C, portA
	   movfp 0x4D, portA
	   movfp 0x4E, portA
	   movfp 0x4F, portA
	   movfp 0x50, portA
	   movfp 0x51, portA
	   movfp 0x52, portA
	   movfp 0x53, portA
	   movfp 0x54, portA
	   movfp 0x55, portA
	   movfp 0x56, portA
	   movfp 0x57, portA
	   movfp 0x58, portA
	   movfp 0x59, portA
	   movfp 0x5A, portA
	   movfp 0x5B, portA
	   movfp 0x5C, portA
	   movfp 0x5D, portA
	   movfp 0x5E, portA

	   return

; Serial port data transfer is not yet implemented

;******************************************************
; start of messages

;   prt_done: dfb "Done !", -1
 
;   prt_process: dfb "Processing. Please Wait !", -1          


;end of messages
;****************************************************** 


	END
